@@ -78,6 +78,7 @@ class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
         super(Plugin, self).__init__(context=context)
         dialog.DialogBaseImpl.__init__(self)    # python super is no good
         self._enabled = False
+        self._question_occurrences = {}
 
     @plugin.event(
         stage=plugin.Stages.STAGE_BOOT,
@@ -185,6 +186,26 @@ class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
                 value = tempval
                 accepted = True
 
+        occurrence = self._question_occurrences.get(name, 1)
+        envkey = '{prefix}{occurrence}/{name}'.format(
+            prefix=constants.CoreEnv.QUESTION_PREFIX,
+            occurrence=str(occurrence),
+            name=name,
+        )
+        self._question_occurrences[name] = occurrence+1
+        if envkey in self.environment:
+            answer = self.environment[envkey]
+            self.dialog.note(text=note, prompt=False)
+            self.dialog.note(
+                _(
+                    'provided answer: {answer}'
+                ).format(
+                    answer=_('(hidden)') if hidden else answer,
+                )
+            )
+            value = answer
+            accepted = True
+
         while not accepted:
             self.dialog.note(text=note, prompt=prompt)
             value = self._readline(hidden=hidden)
@@ -198,6 +219,8 @@ class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
                 self.logger.error(_('Please specify value'))
             else:
                 accepted = True
+                self.environment[envkey] = value
+
         return value
 
     def queryMultiString(self, name, note=None):
