@@ -29,6 +29,10 @@ def _(m):
     return gettext.dgettext(message=m, domain='otopi')
 
 
+def _qep(s):
+    return '%s%s' % (dialogcons.DialogMachineConst.QUERY_EXTRA_PREFIX, s)
+
+
 @util.export
 class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
     """Machine dialog protocol provider.
@@ -61,6 +65,7 @@ class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
         super(Plugin, self).__init__(context=context)
         dialog.DialogBaseImpl.__init__(self)    # python super is no good
         self._enabled = False
+        self._question_occurrences = {}
 
     @plugin.event(
         stage=plugin.Stages.STAGE_BOOT,
@@ -82,6 +87,53 @@ class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
         self._enabled = True
         self.context.registerDialog(self)
 
+    def _machine_dialog_event_callback(self, prefix, stage, method):
+        self._write(
+            text='{prefix} STAGE {stage} METHOD {name} ({givenname})\n'.format(
+                prefix=prefix,
+                stage=plugin.Stages.stage_id(stage),
+                name=self.context.methodName(method),
+                givenname=method['name'],
+            )
+        )
+
+    @plugin.event(
+        stage=plugin.Stages.STAGE_INIT,
+        condition=lambda self: self._enabled,
+    )
+    def _init_machine_events_stuff(self):
+        def _pre(stage, method):
+            self._machine_dialog_event_callback(
+                prefix=_qep(dialogcons.DialogMachineConst.EVENT_START),
+                stage=stage,
+                method=method,
+            )
+        self.context.registerPreEventCallback(_pre)
+
+        def _post(stage, method):
+            self._machine_dialog_event_callback(
+                prefix=_qep(dialogcons.DialogMachineConst.EVENT_END),
+                stage=stage,
+                method=method,
+            )
+        self.context.registerPostEventCallback(_post)
+
+        self._write(
+            text='%s\n' % _qep(dialogcons.DialogMachineConst.EVENTS_LIST_START)
+        )
+        for stage, name, givenname in self.context.getSequence():
+            self._write(
+                text='{p} STAGE {stage} METHOD {name} ({givenname})\n'.format(
+                    p=_qep(dialogcons.DialogMachineConst.EVENTS_LIST_ENTRY),
+                    stage=plugin.Stages.stage_id(stage),
+                    name=name,
+                    givenname=givenname,
+                )
+            )
+        self._write(
+            text='%s\n' % _qep(dialogcons.DialogMachineConst.EVENTS_LIST_END)
+        )
+
     @plugin.event(
         stage=plugin.Stages.STAGE_TERMINATE,
         priority=plugin.Stages.PRIORITY_LAST + 10,
@@ -122,18 +174,16 @@ class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
 
     def _writeQueryStart(self, name):
         self._write(
-            text='%s%s %s\n' % (
-                dialogcons.DialogMachineConst.QUERY_EXTRA_PREFIX,
-                dialogcons.DialogMachineConst.QUERY_START,
+            text='%s %s\n' % (
+                _qep(dialogcons.DialogMachineConst.QUERY_START),
                 name,
             )
         )
 
     def _writeQueryEnd(self, name):
         self._write(
-            text='%s%s %s\n' % (
-                dialogcons.DialogMachineConst.QUERY_EXTRA_PREFIX,
-                dialogcons.DialogMachineConst.QUERY_END,
+            text='%s %s\n' % (
+                _qep(dialogcons.DialogMachineConst.QUERY_END),
                 name,
             )
         )
@@ -159,64 +209,91 @@ class Plugin(plugin.PluginBase, dialog.DialogBaseImpl):
             default=default,
         )
 
-        self._writeQueryStart(name)
-        self.dialog.note(text=note, prompt=prompt)
-        if default:
-            self._write(
-                text='%s%s %s\n' % (
-                    dialogcons.DialogMachineConst.QUERY_EXTRA_PREFIX,
-                    dialogcons.DialogMachineConst.QUERY_DEFAULT_VALUE,
-                    default,
-                )
-            )
-        if validValues:
-            self._write(
-                text='%s%s %s\n' % (
-                    dialogcons.DialogMachineConst.QUERY_EXTRA_PREFIX,
-                    dialogcons.DialogMachineConst.QUERY_VALID_VALUES,
-                    '|'.join(
-                        [
-                            x.replace('\\', '\\\\').replace('|', '\|')
-                            for x in validValues
-                        ]
-                    ),
-                )
-            )
-        self._write(
-            text='%s%s %s\n' % (
-                dialogcons.DialogMachineConst.QUERY_EXTRA_PREFIX,
-                dialogcons.DialogMachineConst.QUERY_HIDDEN,
-                (
-                    dialogcons.DialogMachineConst.QUERY_HIDDEN_TRUE if hidden
-                    else dialogcons.DialogMachineConst.QUERY_HIDDEN_FALSE
-                )
-            )
+        occurrence = self._question_occurrences.get(name, 1)
+        envkey = '{prefix}{occurrence}/{name}'.format(
+            prefix=constants.CoreEnv.QUESTION_PREFIX,
+            occurrence=str(occurrence),
+            name=name,
         )
+        self._question_occurrences[name] = occurrence+1
+        if envkey in self.environment:
+            # Answer provided in answerfile. No need to prompt or
+            # anything. TODO Consider formalizing this in the machine
+            # dialog protocol so that the client knows that we were
+            # supposed to ask something but already got an answer.
+            # For now just output this as a note, just like the
+            # human dialect.
+            answer = self.environment[envkey]
+            self.dialog.note(text=note, prompt=False)
+            self.dialog.note(
+                _(
+                    'provided answer: {answer}'
+                ).format(
+                    answer=_('(hidden)') if hidden else answer,
+                )
+            )
+            value = answer
+        else:
+            self._writeQueryStart(name)
+            self.dialog.note(text=note, prompt=prompt)
+            if default:
+                self._write(
+                    text='%s %s\n' % (
+                        _qep(
+                            dialogcons.DialogMachineConst.QUERY_DEFAULT_VALUE
+                        ),
+                        default,
+                    )
+                )
+            if validValues:
+                self._write(
+                    text='%s %s\n' % (
+                        _qep(dialogcons.DialogMachineConst.QUERY_VALID_VALUES),
+                        '|'.join(
+                            [
+                                x.replace('\\', '\\\\').replace('|', '\|')
+                                for x in validValues
+                            ]
+                        ),
+                    )
+                )
+            self._write(
+                text='%s %s\n' % (
+                    _qep(dialogcons.DialogMachineConst.QUERY_HIDDEN),
+                    (
+                        dialogcons.DialogMachineConst.QUERY_HIDDEN_TRUE
+                        if hidden
+                        else dialogcons.DialogMachineConst.QUERY_HIDDEN_FALSE
+                    )
+                )
+            )
 
-        self._write(
-            text='%s%s %s\n' % (
-                dialogcons.DialogMachineConst.REQUEST_PREFIX,
-                dialogcons.DialogMachineConst.QUERY_STRING,
-                name,
-            )
-        )
-        self._writeQueryEnd(name)
-        if not caseSensitive and validValues is not None:
-            validValues = [v.lower() for v in validValues]
-        value = self._readline(hidden)
-        if not value and default is not None:
-            value = default
-        if not caseSensitive:
-            value = value.lower()
-        if (
-            (validValues is not None and value not in validValues) or
-            (not value and value != default)
-        ):
-            raise RuntimeError(
-                _("Invalid value provided to '{name}'").format(
-                    name=name
+            self._write(
+                text='%s%s %s\n' % (
+                    dialogcons.DialogMachineConst.REQUEST_PREFIX,
+                    dialogcons.DialogMachineConst.QUERY_STRING,
+                    name,
                 )
             )
+            self._writeQueryEnd(name)
+            if not caseSensitive and validValues is not None:
+                validValues = [v.lower() for v in validValues]
+            value = self._readline(hidden)
+            if not value and default is not None:
+                value = default
+            if not caseSensitive:
+                value = value.lower()
+            if (
+                (validValues is not None and value not in validValues) or
+                (not value and value != default)
+            ):
+                raise RuntimeError(
+                    _("Invalid value provided to '{name}'").format(
+                        name=name
+                    )
+                )
+            self.environment[envkey] = value
+
         return value
 
     def queryMultiString(self, name, note=None):
